/** @file triangulierung.h
 * A triangulation of the origami-curve. Each triangle corresponds
 * to a copy of the standard fundamental domain of SL(2,Z).
 * The triangles are glued together by the algorithm for the 
 * computation of the Veech-Group in veechgruppe.h.
 * 
 * @author Gabriela Schmithuesen
 * @version 1.0
 * @date 29.01.2007
 */

/*  This file is part of the Origami-Library, which is a collection
 *  of algorithms that calculate properties of mathematical objects
 *  called "origamis".
 *
 *  Copyright (C) 2007 Gabriela Schmithuesen
 *  Address: Institute for Algebra and Geometry, Fakultaet fuer
 *  Mathematik, Universitaet Karlsruhe, D-76128 Karlsruhe, Germany.
 *  E-mail: schmithuesen@math.uni-karlsruhe.de
 *  or origami@math.uni-karlsruhe.de
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 3
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, see
 *  <http://www.gnu.org/licenses/> or write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA  02110-1301, USA.
 *
 *  If you publish results you obtained by using this software please
 *  don't forget to give appropriate credit in your publication.
 */

#ifndef triangulierung1_cc
#define triangulierung1_cc

#include <iostream>
#include <cassert>

#include "SL2_Z.h"
#include "freeGroups.h"

class d_ecke;

class dreieck
{public:
  dreieck *stranslat, *ttranslat, *tinvtranslat;
  int Awert, Bwert, Cwert; //values at the vertices ("Eckenwerte")
  d_ecke *Aecke, *Becke, *Cecke; //vertices(A at infinity, mahtematically positive)
  bool Aerfasst, Berfasst, Cerfasst; //true, if vertex in "Eckenliste" included
  SL2_Z_ST dreieck_inhalt;

 public:
	dreieck(SL2_Z_ST vt);
	dreieck();

	dreieck *return_ttranslat() 				const {assert(ttranslat!=0); return ttranslat;};
	dreieck *return_stranslat()				const {assert(stranslat!=0); return stranslat;};
	dreieck *return_tinvtranslat()			const {assert(tinvtranslat!=0); return tinvtranslat;};
   SL2_Z_ST return_inhalt()					const {return dreieck_inhalt;};
	void def_ttranslat(dreieck *dd)					{ttranslat = dd; assert(dd!=0);};
	void def_stranslat(dreieck *dd)					{stranslat = dd; assert(dd!=0);};
	void def_tinvtranslat(dreieck *dd)				{tinvtranslat = dd; assert(dd!=0);};
   void def_Aerfasst(bool erf)						{Aerfasst = erf;};
   void def_Berfasst(bool erf)						{Berfasst = erf;};
   void def_Cerfasst(bool erf)						{Cerfasst = erf;};
	d_ecke *return_Aecke()								{assert(Aecke!=0); return Aecke;};
	d_ecke *return_Becke()								{assert(Becke!=0); return Becke;};
	d_ecke *return_Cecke()								{assert(Cecke!=0); return Cecke;};
   bool return_Aerfasst()						const {return Aerfasst;};
   bool return_Berfasst()						const {return Berfasst;};
   bool return_Cerfasst()						const {return Cerfasst;};
   void def_Awert(int n)								{Awert = n;};
   void def_Bwert(int n)								{Bwert = n;};
   void def_Cwert(int n)								{Cwert = n;};
   int return_Awert()							const {return Awert;};
   int return_Bwert()							const {return Bwert;};
   int return_Cwert()							const {return Cwert;};
	void def_Aecke(d_ecke *A)							{Aecke = A; assert(A!=0);};
	void def_Becke(d_ecke *B)							{Becke = B; assert(B!=0);};
	void def_Cecke(d_ecke *C)							{Cecke = C; assert(C!=0);};
   void ausgabe_dreieck_inhalt()				const {cout << dreieck_inhalt;};
	void ausgabe()									const	{cout << dreieck_inhalt;};
	void ausgabe_more1() const;
};




class d_ecke
{private:
  int ordnung;
  int nummer;
  bool unendlich; //true = vertex at infinity; false = not at infinity
 public:
  list<dreieck*> dl;

 public:
  d_ecke(int n, bool unendl)			{nummer = n;ordnung = 0;unendlich = unendl;};
  d_ecke(int n)							{nummer = n;ordnung = 0;};
  d_ecke()									{ordnung = 0;}

  void ausgabe() const;
  void einfuegen (dreieck *dd)		{dl.push_back(dd); ordnung++; assert(dd!=0);};
  bool return_unendlich()		const {return unendlich;};
  int return_ordnung()			const {return ordnung;};

};




class  triang
{public:

   list<d_ecke> el;
   int eckenzahl;
   int flaechenzahl;
   int sondereckenzahl;
   list<dreieck> fl;
 public:
	 typedef list<dreieck>::iterator iterator;   // type(*it) = dreieck
	 typedef list<dreieck>::const_iterator const_iterator;   // type(*it) = dreieck
	 iterator begin()							{return fl.begin();}
	 iterator end()  							{return fl.end();}
	 const_iterator begin()	const			{return fl.begin();}
	 const_iterator end()  const			{return fl.end();}

	triang(const dreieck& d);
   triang();
   int return_eckenzahl() const  		{return eckenzahl;};
   int return_flaechenzahl() const 		{return fl.size();};
   int return_sondereckenzahl() const	{return sondereckenzahl;};

  //"ausgabe" print the list of matrices "Matrixliste":
	void ausgabe() const;
	void ausgabe_kurz() const;
	void ausgabe_ecken() const; //Neu!!! (28.8.2003)
  

  // Prints the cusps of the triangulation: 
  // for each cusp: amplitude,  a corresponding parabolic element
  // and the direction
	void ausgabe_spitzen() const; //Neu!!! (19.12.2003)
	
	void ausgabe_flaechen_more1() const; //Neu!!! (1.9.2003)

  
   //"anfuegen" attaches the triangel d_neu to the triangle d_alt 
   //at the edge translattyp. (0 = S, 1 = T, 3 = T_inv )
	dreieck* anfuegen(int translattyp, const dreieck& d_neu, dreieck* dd_alt);
	dreieck* anfuegen(int translattyp, const dreieck& d_neu, iterator dd_alt)
		 {return anfuegen(translattyp,d_neu,&*dd_alt);}

   //"verkleben" glues the triangles d_ausgang and d_ziel along the 
   //edge in direction translattyp (as in anfuegen) of the triangle d_ausgang
	void verkleben (int translattyp, dreieck* dd_ausgang, dreieck* dd_ziel);
	void verkleben (int translattyp, iterator dd_ausgang, iterator dd_ziel) 
		 {verkleben (translattyp, &*dd_ausgang, &*dd_ziel);}
	
   //"ecken_bestimmen" determines for the triangulated surface 
   //the list of vertices "Eckenliste" as follows:
   //It determines iteratively for each mmarked vertex of the triangle (erfasst = false)
   //all vertices, which are glued to this vertex.
	void ecken_bestimmen ();

	int g() const;

	//pkt_anzahl returns the number of cusps (i.e. verteces at infinity)
	int pkt_anzahl() const;
};

#endif /*triangulierung1_cc*/
